# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class XdgPositioner(Interface):
    """Child surface positioner

    The :class:`XdgPositioner` provides a collection of rules for the placement
    of a child surface relative to a parent surface. Rules can be defined to
    ensure the child surface remains within the visible area's borders, and to
    specify how the child surface changes its position, such as sliding along
    an axis, or flipping around a rectangle. These positioner-created rules are
    constrained by the requirement that a child surface must intersect with or
    be at least partially adjacent to its parent surface.

    See the various requests for details about possible rules.

    At the time of the request, the compositor makes a copy of the rules
    specified by the :class:`XdgPositioner`. Thus, after the request is
    complete the :class:`XdgPositioner` object can be destroyed or reused;
    further changes to the object will have no effect on previous usages.

    For an :class:`XdgPositioner` object to be considered complete, it must
    have a non-zero size set by set_size, and a non-zero anchor rectangle set
    by set_anchor_rect. Passing an incomplete :class:`XdgPositioner` object
    when positioning a surface raises an error.
    """

    name = "xdg_positioner"
    version = 3

    class error(enum.IntEnum):
        invalid_input = 0

    class anchor(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 3
        right = 4
        top_left = 5
        bottom_left = 6
        top_right = 7
        bottom_right = 8

    class gravity(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 3
        right = 4
        top_left = 5
        bottom_left = 6
        top_right = 7
        bottom_right = 8

    class constraint_adjustment(enum.IntFlag):
        none = 0
        slide_x = 1
        slide_y = 2
        flip_x = 4
        flip_y = 8
        resize_x = 16
        resize_y = 32


class XdgPositionerProxy(Proxy):
    interface = XdgPositioner

    @XdgPositioner.request()
    def destroy(self):
        """Destroy the :class:`XdgPositioner` object

        Notify the compositor that the :class:`XdgPositioner` will no longer be
        used.
        """
        self._marshal(0)
        self._destroy()

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_size(self, width, height):
        """Set the size of the to-be positioned rectangle

        Set the size of the surface that is to be positioned with the
        positioner object. The size is in surface-local coordinates and
        corresponds to the window geometry. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        If a zero or negative size is set the invalid_input error is raised.

        :param width:
            width of positioned rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of positioned rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(1, width, height)

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_anchor_rect(self, x, y, width, height):
        """Set the anchor rectangle within the parent surface

        Specify the anchor rectangle within the parent surface that the child
        surface will be placed relative to. The rectangle is relative to the
        window geometry as defined by :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>` of the
        parent surface.

        When the :class:`XdgPositioner` object is used to position a child
        surface, the anchor rectangle may not extend outside the window
        geometry of the positioned child's parent surface.

        If a negative size is set the invalid_input error is raised.

        :param x:
            x position of anchor rectangle
        :type x:
            `ArgumentType.Int`
        :param y:
            y position of anchor rectangle
        :type y:
            `ArgumentType.Int`
        :param width:
            width of anchor rectangle
        :type width:
            `ArgumentType.Int`
        :param height:
            height of anchor rectangle
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(2, x, y, width, height)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
    )
    def set_anchor(self, anchor):
        """Set anchor rectangle anchor

        Defines the anchor point for the anchor rectangle. The specified anchor
        is used derive an anchor point that the child surface will be
        positioned relative to. If a corner anchor is set (e.g. 'top_left' or
        'bottom_right'), the anchor point will be at the specified corner;
        otherwise, the derived anchor point will be centered on the specified
        edge, or in the center of the anchor rectangle if no edge is specified.

        :param anchor:
            anchor
        :type anchor:
            `ArgumentType.Uint`
        """
        self._marshal(3, anchor)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
    )
    def set_gravity(self, gravity):
        """Set child surface gravity

        Defines in what direction a surface should be positioned, relative to
        the anchor point of the parent surface. If a corner gravity is
        specified (e.g. 'bottom_right' or 'top_left'), then the child surface
        will be placed towards the specified gravity; otherwise, the child
        surface will be centered over the anchor point on any axis that had no
        gravity specified.

        :param gravity:
            gravity direction
        :type gravity:
            `ArgumentType.Uint`
        """
        self._marshal(4, gravity)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
    )
    def set_constraint_adjustment(self, constraint_adjustment):
        """Set the adjustment to be done when constrained

        Specify how the window should be positioned if the originally intended
        position caused the surface to be constrained, meaning at least
        partially outside positioning boundaries set by the compositor. The
        adjustment is set by constructing a bitmask describing the adjustment
        to be made when the surface is constrained on that axis.

        If no bit for one axis is set, the compositor will assume that the
        child surface should not change its position on that axis when
        constrained.

        If more than one bit for one axis is set, the order of how adjustments
        are applied is specified in the corresponding adjustment descriptions.

        The default adjustment is none.

        :param constraint_adjustment:
            bit mask of constraint adjustments
        :type constraint_adjustment:
            `ArgumentType.Uint`
        """
        self._marshal(5, constraint_adjustment)

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_offset(self, x, y):
        """Set surface position offset

        Specify the surface position offset relative to the position of the
        anchor on the anchor rectangle and the anchor on the surface. For
        example if the anchor of the anchor rectangle is at (x, y), the surface
        has the gravity bottom|right, and the offset is (ox, oy), the
        calculated surface position will be (x + ox, y + oy). The offset
        position of the surface is the one used for constraint testing. See
        set_constraint_adjustment.

        An example use case is placing a popup menu on top of a user interface
        element, while aligning the user interface element of the parent
        surface with some user interface element placed somewhere in the popup
        surface.

        :param x:
            surface position x offset
        :type x:
            `ArgumentType.Int`
        :param y:
            surface position y offset
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(6, x, y)

    @XdgPositioner.request(version=3)
    def set_reactive(self):
        """Continuously reconstrain the surface

        When set reactive, the surface is reconstrained if the conditions used
        for constraining changed, e.g. the parent window moved.

        If the conditions changed and the popup was reconstrained, an
        :func:`XdgPopup.configure()
        <pywayland.protocol.xdg_shell.XdgPopup.configure>` event is sent with
        updated geometry, followed by an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event.
        """
        self._marshal(7)

    @XdgPositioner.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        version=3,
    )
    def set_parent_size(self, parent_width, parent_height):
        """

        Set the parent window geometry the compositor should use when
        positioning the popup. The compositor may use this information to
        determine the future state the popup should be constrained using. If
        this doesn't match the dimension of the parent the popup is eventually
        positioned against, the behavior is undefined.

        The arguments are given in the surface-local coordinate space.

        :param parent_width:
            future window geometry width of parent
        :type parent_width:
            `ArgumentType.Int`
        :param parent_height:
            future window geometry height of parent
        :type parent_height:
            `ArgumentType.Int`
        """
        self._marshal(8, parent_width, parent_height)

    @XdgPositioner.request(
        Argument(ArgumentType.Uint),
        version=3,
    )
    def set_parent_configure(self, serial):
        """Set parent configure this is a response to

        Set the serial of an :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` event this
        positioner will be used in response to. The compositor may use this
        information together with set_parent_size to determine what future
        state the popup should be constrained using.

        :param serial:
            serial of parent configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(9, serial)


class XdgPositionerResource(Resource):
    interface = XdgPositioner


class XdgPositionerGlobal(Global):
    interface = XdgPositioner


XdgPositioner._gen_c()
XdgPositioner.proxy_class = XdgPositionerProxy
XdgPositioner.resource_class = XdgPositionerResource
XdgPositioner.global_class = XdgPositionerGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015-2016 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface


class ZxdgImportedV2(Interface):
    """An imported surface handle

    An xdg_imported object represents an imported reference to surface exported
    by some client. A client can use this interface to manipulate relationships
    between its own surfaces and the imported surface.
    """

    name = "zxdg_imported_v2"
    version = 1

    class error(enum.IntEnum):
        invalid_surface = 0


class ZxdgImportedV2Proxy(Proxy):
    interface = ZxdgImportedV2

    @ZxdgImportedV2.request()
    def destroy(self):
        """Destroy the xdg_imported object

        Notify the compositor that it will no longer use the xdg_imported
        object. Any relationship that may have been set up will at this point
        be invalidated.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgImportedV2.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def set_parent_of(self, surface):
        """Set as the parent of some surface

        Set the imported surface as the parent of some surface of the client.
        The passed surface must be an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` equivalent,
        otherwise an invalid_surface protocol error is sent. Calling this
        function sets up a surface to surface relation with the same stacking
        and positioning semantics as :func:`XdgToplevel.set_parent()
        <pywayland.protocol.xdg_shell.XdgToplevel.set_parent>`.

        :param surface:
            the child surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(1, surface)


class ZxdgImportedV2Resource(Resource):
    interface = ZxdgImportedV2

    @ZxdgImportedV2.event()
    def destroyed(self):
        """The imported surface handle has been destroyed

        The imported surface handle has been destroyed and any relationship set
        up has been invalidated. This may happen for various reasons, for
        example if the exported surface or the exported surface handle has been
        destroyed, if the handle used for importing was invalid.
        """
        self._post_event(0)


class ZxdgImportedV2Global(Global):
    interface = ZxdgImportedV2


ZxdgImportedV2._gen_c()
ZxdgImportedV2.proxy_class = ZxdgImportedV2Proxy
ZxdgImportedV2.resource_class = ZxdgImportedV2Resource
ZxdgImportedV2.global_class = ZxdgImportedV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2012, 2013 Intel Corporation
# Copyright © 2015, 2016 Jan Arne Petersen
# Copyright © 2017, 2018 Red Hat, Inc.
# Copyright © 2018       Purism SPC
#
# Permission to use, copy, modify, distribute, and sell this
# software and its documentation for any purpose is hereby granted
# without fee, provided that the above copyright notice appear in
# all copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of
# the copyright holders not be used in advertising or publicity
# pertaining to distribution of the software without specific,
# written prior permission.  The copyright holders make no
# representations about the suitability of this software for any
# purpose.  It is provided "as is" without express or implied
# warranty.
#
# THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
# SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
# SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
# AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
# ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
# THIS SOFTWARE.

from .zwp_text_input_manager_v3 import ZwpTextInputManagerV3  # noqa: F401
from .zwp_text_input_v3 import ZwpTextInputV3  # noqa: F401



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015 Samsung Electronics Co., Ltd
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .zwp_idle_inhibitor_v1 import ZwpIdleInhibitorV1


class ZwpIdleInhibitManagerV1(Interface):
    """Control behavior when display idles

    This interface permits inhibiting the idle behavior such as screen
    blanking, locking, and screensaving.  The client binds the idle manager
    globally, then creates idle-inhibitor objects for each surface.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_idle_inhibit_manager_v1"
    version = 1


class ZwpIdleInhibitManagerV1Proxy(Proxy):
    interface = ZwpIdleInhibitManagerV1

    @ZwpIdleInhibitManagerV1.request()
    def destroy(self):
        """Destroy the idle inhibitor object

        Destroy the inhibit manager.
        """
        self._marshal(0)
        self._destroy()

    @ZwpIdleInhibitManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpIdleInhibitorV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def create_inhibitor(self, surface):
        """Create a new inhibitor object

        Create a new inhibitor object associated with the given surface.

        :param surface:
            the surface that inhibits the idle behavior
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.idle_inhibit_unstable_v1.ZwpIdleInhibitorV1`
        """
        id = self._marshal_constructor(1, ZwpIdleInhibitorV1, surface)
        return id


class ZwpIdleInhibitManagerV1Resource(Resource):
    interface = ZwpIdleInhibitManagerV1


class ZwpIdleInhibitManagerV1Global(Global):
    interface = ZwpIdleInhibitManagerV1


ZwpIdleInhibitManagerV1._gen_c()
ZwpIdleInhibitManagerV1.proxy_class = ZwpIdleInhibitManagerV1Proxy
ZwpIdleInhibitManagerV1.resource_class = ZwpIdleInhibitManagerV1Resource
ZwpIdleInhibitManagerV1.global_class = ZwpIdleInhibitManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2016 Yong Bakos
# Copyright © 2015 Jason Ekstrand
# Copyright © 2015 Jonas Ådahl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlOutput
from ..wayland import WlSurface
from .zwp_fullscreen_shell_mode_feedback_v1 import ZwpFullscreenShellModeFeedbackV1


class ZwpFullscreenShellV1(Interface):
    """Displays a single surface per output

    Displays a single surface per output.

    This interface provides a mechanism for a single client to display simple
    full-screen surfaces.  While there technically may be multiple clients
    bound to this interface, only one of those clients should be shown at a
    time.

    To present a surface, the client uses either the present_surface or
    present_surface_for_mode requests.  Presenting a surface takes effect on
    the next :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>`.  See the individual
    requests for details about scaling and mode switches.

    The client can have at most one surface per output at any time. Requesting
    a surface to be presented on an output that already has a surface replaces
    the previously presented surface.  Presenting a null surface removes its
    content and effectively disables the output. Exactly what happens when an
    output is "disabled" is compositor-specific.  The same surface may be
    presented on multiple outputs simultaneously.

    Once a surface is presented on an output, it stays on that output until
    either the client removes it or the compositor destroys the output.  This
    way, the client can update the output's contents by simply attaching a new
    buffer.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zwp_fullscreen_shell_v1"
    version = 1

    class capability(enum.IntEnum):
        arbitrary_modes = 1
        cursor_plane = 2

    class present_method(enum.IntEnum):
        default = 0
        center = 1
        zoom = 2
        zoom_crop = 3
        stretch = 4

    class error(enum.IntEnum):
        invalid_method = 0
        role = 1


class ZwpFullscreenShellV1Proxy(Proxy):
    interface = ZwpFullscreenShellV1

    @ZwpFullscreenShellV1.request()
    def release(self):
        """Release the wl_fullscreen_shell interface

        Release the binding from the wl_fullscreen_shell interface.

        This destroys the server-side object and frees this binding.  If the
        client binds to wl_fullscreen_shell multiple times, it may wish to free
        some of those bindings.
        """
        self._marshal(0)
        self._destroy()

    @ZwpFullscreenShellV1.request(
        Argument(ArgumentType.Object, interface=WlSurface, nullable=True),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def present_surface(self, surface, method, output):
        """Present surface for display

        Present a surface on the given output.

        If the output is null, the compositor will present the surface on
        whatever display (or displays) it thinks best.  In particular, this may
        replace any or all surfaces currently presented so it should not be
        used in combination with placing surfaces on specific outputs.

        The method parameter is a hint to the compositor for how the surface is
        to be presented.  In particular, it tells the compositor how to handle
        a size mismatch between the presented surface and the output.  The
        compositor is free to ignore this parameter.

        The "zoom", "zoom_crop", and "stretch" methods imply a scaling
        operation on the surface.  This will override any kind of output
        scaling, so the buffer_scale property of the surface is effectively
        ignored.

        This request gives the surface the role of a fullscreen shell surface.
        If the surface already has another role, it raises a role protocol
        error.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface` or `None`
        :param method:
        :type method:
            `ArgumentType.Uint`
        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(1, surface, method, output)

    @ZwpFullscreenShellV1.request(
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlOutput),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.NewId, interface=ZwpFullscreenShellModeFeedbackV1),
    )
    def present_surface_for_mode(self, surface, output, framerate):
        """Present surface for display at a particular mode

        Presents a surface on the given output for a particular mode.

        If the current size of the output differs from that of the surface, the
        compositor will attempt to change the size of the output to match the
        surface.  The result of the mode-switch operation will be returned via
        the provided wl_fullscreen_shell_mode_feedback object.

        If the current output mode matches the one requested or if the
        compositor successfully switches the mode to match the surface, then
        the mode_successful event will be sent and the output will contain the
        contents of the given surface.  If the compositor cannot match the
        output size to the surface size, the mode_failed will be sent and the
        output will contain the contents of the previously presented surface
        (if any).  If another surface is presented on the given output before
        either of these has a chance to happen, the present_cancelled event
        will be sent.

        Due to race conditions and other issues unknown to the client, no mode-
        switch operation is guaranteed to succeed.  However, if the mode is one
        advertised by :func:`WlOutput.mode()
        <pywayland.protocol.wayland.WlOutput.mode>` or if the compositor
        advertises the ARBITRARY_MODES capability, then the client should
        expect that the mode-switch operation will usually succeed.

        If the size of the presented surface changes, the resulting output is
        undefined.  The compositor may attempt to change the output mode to
        compensate.  However, there is no guarantee that a suitable mode will
        be found and the client has no way to be notified of success or
        failure.

        The framerate parameter specifies the desired framerate for the output
        in mHz.  The compositor is free to ignore this parameter.  A value of 0
        indicates that the client has no preference.

        If the value of :func:`WlOutput.scale()
        <pywayland.protocol.wayland.WlOutput.scale>` differs from
        :func:`WlSurface.buffer_scale()
        <pywayland.protocol.wayland.WlSurface.buffer_scale>`, then the
        compositor may choose a mode that matches either the buffer size or the
        surface size.  In either case, the surface will fill the output.

        This request gives the surface the role of a fullscreen shell surface.
        If the surface already has another role, it raises a role protocol
        error.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        :param framerate:
        :type framerate:
            `ArgumentType.Int`
        :returns:
            :class:`~pywayland.protocol.fullscreen_shell_unstable_v1.ZwpFullscreenShellModeFeedbackV1`
        """
        feedback = self._marshal_constructor(2, ZwpFullscreenShellModeFeedbackV1, surface, output, framerate)
        return feedback


class ZwpFullscreenShellV1Resource(Resource):
    interface = ZwpFullscreenShellV1

    @ZwpFullscreenShellV1.event(
        Argument(ArgumentType.Uint),
    )
    def capability(self, capability):
        """Advertises a capability of the compositor

        Advertises a single capability of the compositor.

        When the wl_fullscreen_shell interface is bound, this event is emitted
        once for each capability advertised.  Valid capabilities are given by
        the wl_fullscreen_shell.capability enum.  If clients want to take
        advantage of any of these capabilities, they should use a
        :func:`WlDisplay.sync() <pywayland.protocol.wayland.WlDisplay.sync>`
        request immediately after binding to ensure that they receive all the
        capability events.

        :param capability:
        :type capability:
            `ArgumentType.Uint`
        """
        self._post_event(0, capability)


class ZwpFullscreenShellV1Global(Global):
    interface = ZwpFullscreenShellV1


ZwpFullscreenShellV1._gen_c()
ZwpFullscreenShellV1.proxy_class = ZwpFullscreenShellV1Proxy
ZwpFullscreenShellV1.resource_class = ZwpFullscreenShellV1Resource
ZwpFullscreenShellV1.global_class = ZwpFullscreenShellV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .zwp_primary_selection_offer_v1 import ZwpPrimarySelectionOfferV1
from .zwp_primary_selection_source_v1 import ZwpPrimarySelectionSourceV1


class ZwpPrimarySelectionDeviceV1(Interface):
    name = "zwp_primary_selection_device_v1"
    version = 1


class ZwpPrimarySelectionDeviceV1Proxy(Proxy):
    interface = ZwpPrimarySelectionDeviceV1

    @ZwpPrimarySelectionDeviceV1.request(
        Argument(ArgumentType.Object, interface=ZwpPrimarySelectionSourceV1, nullable=True),
        Argument(ArgumentType.Uint),
    )
    def set_selection(self, source, serial):
        """Set the primary selection

        Replaces the current selection. The previous owner of the primary
        selection will receive a wp_primary_selection_source.cancelled event.

        To unset the selection, set the source to NULL.

        :param source:
        :type source:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionSourceV1` or `None`
        :param serial:
            serial of the event that triggered this request
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, source, serial)

    @ZwpPrimarySelectionDeviceV1.request()
    def destroy(self):
        """Destroy the primary selection device

        Destroy the primary selection device.
        """
        self._marshal(1)
        self._destroy()


class ZwpPrimarySelectionDeviceV1Resource(Resource):
    interface = ZwpPrimarySelectionDeviceV1

    @ZwpPrimarySelectionDeviceV1.event(
        Argument(ArgumentType.NewId, interface=ZwpPrimarySelectionOfferV1),
    )
    def data_offer(self, offer):
        """Introduce a new wp_primary_selection_offer

        Introduces a new wp_primary_selection_offer object that may be used to
        receive the current primary selection. Immediately following this
        event, the new wp_primary_selection_offer object will send
        wp_primary_selection_offer.offer events to describe the offered mime
        types.

        :param offer:
        :type offer:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionOfferV1`
        """
        self._post_event(0, offer)

    @ZwpPrimarySelectionDeviceV1.event(
        Argument(ArgumentType.Object, interface=ZwpPrimarySelectionOfferV1, nullable=True),
    )
    def selection(self, id):
        """Advertise a new primary selection

        The wp_primary_selection_device.selection event is sent to notify the
        client of a new primary selection. This event is sent after the
        wp_primary_selection.data_offer event introducing this object, and
        after the offer has announced its mimetypes through
        wp_primary_selection_offer.offer.

        The data_offer is valid until a new offer or NULL is received or until
        the client loses keyboard focus. The client must destroy the previous
        selection data_offer, if any, upon receiving this event.

        :param id:
        :type id:
            :class:`~pywayland.protocol.wp_primary_selection_unstable_v1.ZwpPrimarySelectionOfferV1` or `None`
        """
        self._post_event(1, id)


class ZwpPrimarySelectionDeviceV1Global(Global):
    interface = ZwpPrimarySelectionDeviceV1


ZwpPrimarySelectionDeviceV1._gen_c()
ZwpPrimarySelectionDeviceV1.proxy_class = ZwpPrimarySelectionDeviceV1Proxy
ZwpPrimarySelectionDeviceV1.resource_class = ZwpPrimarySelectionDeviceV1Resource
ZwpPrimarySelectionDeviceV1.global_class = ZwpPrimarySelectionDeviceV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from ..wayland import WlSurface
from .zwp_xwayland_keyboard_grab_v1 import ZwpXwaylandKeyboardGrabV1


class ZwpXwaylandKeyboardGrabManagerV1(Interface):
    """Context object for keyboard grab manager

    A global interface used for grabbing the keyboard.
    """

    name = "zwp_xwayland_keyboard_grab_manager_v1"
    version = 1


class ZwpXwaylandKeyboardGrabManagerV1Proxy(Proxy):
    interface = ZwpXwaylandKeyboardGrabManagerV1

    @ZwpXwaylandKeyboardGrabManagerV1.request()
    def destroy(self):
        """Destroy the keyboard grab manager

        Destroy the keyboard grab manager.
        """
        self._marshal(0)
        self._destroy()

    @ZwpXwaylandKeyboardGrabManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZwpXwaylandKeyboardGrabV1),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def grab_keyboard(self, surface, seat):
        """Grab the keyboard to a surface

        The grab_keyboard request asks for a grab of the keyboard, forcing the
        keyboard focus for the given seat upon the given surface.

        The protocol provides no guarantee that the grab is ever satisfied, and
        does not require the compositor to send an error if the grab cannot
        ever be satisfied. It is thus possible to request a keyboard grab that
        will never be effective.

        The protocol:

        * does not guarantee that the grab itself is applied for a surface, the
          grab request may be silently ignored by the compositor,

        * does not guarantee that any events are sent to this client even if
          the grab is applied to a surface,

        * does not guarantee that events sent to this client are exhaustive, a
          compositor may filter some events for its own consumption,

        * does not guarantee that events sent to this client are continuous, a
          compositor may change and reroute keyboard events while the grab is
          nominally active.

        :param surface:
            surface to report keyboard events to
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param seat:
            the seat for which the keyboard should be grabbed
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.xwayland_keyboard_grab_unstable_v1.ZwpXwaylandKeyboardGrabV1`
        """
        id = self._marshal_constructor(1, ZwpXwaylandKeyboardGrabV1, surface, seat)
        return id


class ZwpXwaylandKeyboardGrabManagerV1Resource(Resource):
    interface = ZwpXwaylandKeyboardGrabManagerV1


class ZwpXwaylandKeyboardGrabManagerV1Global(Global):
    interface = ZwpXwaylandKeyboardGrabManagerV1


ZwpXwaylandKeyboardGrabManagerV1._gen_c()
ZwpXwaylandKeyboardGrabManagerV1.proxy_class = ZwpXwaylandKeyboardGrabManagerV1Proxy
ZwpXwaylandKeyboardGrabManagerV1.resource_class = ZwpXwaylandKeyboardGrabManagerV1Resource
ZwpXwaylandKeyboardGrabManagerV1.global_class = ZwpXwaylandKeyboardGrabManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2014      Jonas Ådahl
# Copyright © 2015      Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlRegion


class ZwpConfinedPointerV1(Interface):
    """Confined pointer object

    The wp_confined_pointer interface represents a confined pointer state.

    This object will send the event 'confined' when the confinement is
    activated. Whenever the confinement is activated, it is guaranteed that the
    surface the pointer is confined to will already have received pointer focus
    and that the pointer will be within the region passed to the request
    creating this object. It is up to the compositor to decide whether this
    requires some user interaction and if the pointer will warp to within the
    passed region if outside.

    To unconfine the pointer, send the destroy request. This will also destroy
    the wp_confined_pointer object.

    If the compositor decides to unconfine the pointer the unconfined event is
    sent. The wp_confined_pointer object is at this point defunct and should be
    destroyed.
    """

    name = "zwp_confined_pointer_v1"
    version = 1


class ZwpConfinedPointerV1Proxy(Proxy):
    interface = ZwpConfinedPointerV1

    @ZwpConfinedPointerV1.request()
    def destroy(self):
        """Destroy the confined pointer object

        Destroy the confined pointer object. If applicable, the compositor will
        unconfine the pointer.
        """
        self._marshal(0)
        self._destroy()

    @ZwpConfinedPointerV1.request(
        Argument(ArgumentType.Object, interface=WlRegion, nullable=True),
    )
    def set_region(self, region):
        """Set a new confine region

        Set a new region used to confine the pointer.

        The new confine region is double-buffered. The new confine region will
        only take effect when the associated surface gets its pending state
        applied. See :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` for details.

        If the confinement is active when the new confinement region is applied
        and the pointer ends up outside of newly applied region, the pointer
        may warped to a position within the new confinement region. If warped,
        a :func:`WlPointer.motion()
        <pywayland.protocol.wayland.WlPointer.motion>` event will be emitted,
        but no wp_relative_pointer.relative_motion event.

        The compositor may also, instead of using the new region, unconfine the
        pointer.

        For details about the confine region, see wp_confined_pointer.

        :param region:
            region of surface
        :type region:
            :class:`~pywayland.protocol.wayland.WlRegion` or `None`
        """
        self._marshal(1, region)


class ZwpConfinedPointerV1Resource(Resource):
    interface = ZwpConfinedPointerV1

    @ZwpConfinedPointerV1.event()
    def confined(self):
        """Pointer confined

        Notification that the pointer confinement of the seat's pointer is
        activated.
        """
        self._post_event(0)

    @ZwpConfinedPointerV1.event()
    def unconfined(self):
        """Pointer unconfined

        Notification that the pointer confinement of the seat's pointer is no
        longer active. If this is a oneshot pointer confinement (see
        wp_pointer_constraints.lifetime) this object is now defunct and should
        be destroyed. If this is a persistent pointer confinement (see
        wp_pointer_constraints.lifetime) this pointer confinement may again
        reactivate in the future.
        """
        self._post_event(1)


class ZwpConfinedPointerV1Global(Global):
    interface = ZwpConfinedPointerV1


ZwpConfinedPointerV1._gen_c()
ZwpConfinedPointerV1.proxy_class = ZwpConfinedPointerV1Proxy
ZwpConfinedPointerV1.resource_class = ZwpConfinedPointerV1Resource
ZwpConfinedPointerV1.global_class = ZwpConfinedPointerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class WlDestructor(Interface):
    """Destructor object

    An interface object with a destructor request.

    And a multiline description.
    """

    name = "wl_destructor"
    version = 1


class WlDestructorProxy(Proxy):
    interface = WlDestructor

    @WlDestructor.request(
        Argument(ArgumentType.NewId, interface=WlDestructor),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Uint),
    )
    def create_interface(self, x, y, width, height, format):
        """Create another interface

        Create a :class:`WlDestructor` interface object

        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        :param format:
        :type format:
            `ArgumentType.Uint`
        :returns:
            :class:`WlDestructor`
        """
        id = self._marshal_constructor(0, WlDestructor, x, y, width, height, format)
        return id

    @WlDestructor.request()
    def destroy(self):
        """Destroy the interface

        Destroy the created interface.
        """
        self._marshal(1)
        self._destroy()


class WlDestructorResource(Resource):
    interface = WlDestructor


class WlDestructorGlobal(Global):
    interface = WlDestructor


WlDestructor._gen_c()
WlDestructor.proxy_class = WlDestructorProxy
WlDestructor.resource_class = WlDestructorResource
WlDestructor.global_class = WlDestructorGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
# Copyright © 2015-2017 Samsung Electronics Co., Ltd
# Copyright © 2015-2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlOutput
from ..wayland import WlSeat


class XdgToplevel(Interface):
    """Toplevel surface

    This interface defines an :class:`~pywayland.protocol.xdg_shell.XdgSurface`
    role which allows a surface to, among other things, set window-like
    properties such as maximize, fullscreen, and minimize, set application-
    specific metadata like title and id, and well as trigger user interactive
    operations such as interactive resize and move.

    Unmapping an :class:`XdgToplevel` means that the surface cannot be shown by
    the compositor until it is explicitly mapped again. All active operations
    (e.g., move, resize) are canceled and all attributes (e.g. title, state,
    stacking, ...) are discarded for an :class:`XdgToplevel` surface when it is
    unmapped. The :class:`XdgToplevel` returns to the state it had right after
    :func:`XdgSurface.get_toplevel()
    <pywayland.protocol.xdg_shell.XdgSurface.get_toplevel>`. The client can re-
    map the toplevel by perfoming a commit without any buffer attached, waiting
    for a configure event and handling it as usual (see
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` description).

    Attaching a null buffer to a toplevel unmaps the surface.
    """

    name = "xdg_toplevel"
    version = 3

    class resize_edge(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 4
        top_left = 5
        bottom_left = 6
        right = 8
        top_right = 9
        bottom_right = 10

    class state(enum.IntEnum):
        maximized = 1
        fullscreen = 2
        resizing = 3
        activated = 4
        tiled_left = 5
        tiled_right = 6
        tiled_top = 7
        tiled_bottom = 8


class XdgToplevelProxy(Proxy):
    interface = XdgToplevel

    @XdgToplevel.request()
    def destroy(self):
        """Destroy the :class:`XdgToplevel`

        This request destroys the role surface and unmaps the surface; see
        "Unmapping" behavior in interface section for details.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=XdgToplevel, nullable=True),
    )
    def set_parent(self, parent):
        """Set the parent of this surface

        Set the "parent" of this surface. This surface should be stacked above
        the parent surface and all other ancestor surfaces.

        Parent windows should be set on dialogs, toolboxes, or other
        "auxiliary" surfaces, so that the parent is raised when the dialog is
        raised.

        Setting a null parent for a child window removes any parent-child
        relationship for the child. Setting a null parent for a window which
        currently has no parent is a no-op.

        If the parent is unmapped then its children are managed as though the
        parent of the now-unmapped parent has become the parent of this
        surface. If no parent exists for the now-unmapped parent then the
        children are managed as though they have no parent surface.

        :param parent:
        :type parent:
            :class:`XdgToplevel` or `None`
        """
        self._marshal(1, parent)

    @XdgToplevel.request(
        Argument(ArgumentType.String),
    )
    def set_title(self, title):
        """Set surface title

        Set a short title for the surface.

        This string may be used to identify the surface in a task bar, window
        list, or other user interface elements provided by the compositor.

        The string must be encoded in UTF-8.

        :param title:
        :type title:
            `ArgumentType.String`
        """
        self._marshal(2, title)

    @XdgToplevel.request(
        Argument(ArgumentType.String),
    )
    def set_app_id(self, app_id):
        """Set application id

        Set an application identifier for the surface.

        The app ID identifies the general class of applications to which the
        surface belongs. The compositor can use this to group multiple surfaces
        together, or to determine how to launch a new application.

        For D-Bus activatable applications, the app ID is used as the D-Bus
        service name.

        The compositor shell will try to group application surfaces together by
        their app ID. As a best practice, it is suggested to select app ID's
        that match the basename of the application's .desktop file. For
        example, "org.freedesktop.FooViewer" where the .desktop file is
        "org.freedesktop.FooViewer.desktop".

        Like other properties, a set_app_id request can be sent after the
        :class:`XdgToplevel` has been mapped to update the property.

        See the desktop-entry specification [0] for more details on application
        identifiers and how they relate to well-known D-Bus names and .desktop
        files.

        [0] http://standards.freedesktop.org/desktop-entry-spec/

        :param app_id:
        :type app_id:
            `ArgumentType.String`
        """
        self._marshal(3, app_id)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def show_window_menu(self, seat, serial, x, y):
        """Show the window menu

        Clients implementing client-side decorations might want to show a
        context menu when right-clicking on the decorations, giving the user a
        menu that they can use to maximize or minimize the window.

        This request asks the compositor to pop up such a window menu at the
        given position, relative to the local surface coordinates of the parent
        surface. There are no guarantees as to what menu items the window menu
        contains.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param x:
            the x position to pop up the window menu at
        :type x:
            `ArgumentType.Int`
        :param y:
            the y position to pop up the window menu at
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(4, seat, serial, x, y)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def move(self, seat, serial):
        """Start an interactive move

        Start an interactive, user-driven move of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive move (touch, pointer, etc).

        The server may ignore move requests depending on the state of the
        surface (e.g. fullscreen or maximized), or if the passed serial is no
        longer valid.

        If triggered, the surface will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the move.
        It is up to the compositor to visually indicate that the move is taking
        place, such as updating a pointer cursor, during the move. There is no
        guarantee that the device focus will return when the move is completed.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(5, seat, serial)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def resize(self, seat, serial, edges):
        """Start an interactive resize

        Start a user-driven, interactive resize of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive resize (touch, pointer, etc).

        The server may ignore resize requests depending on the state of the
        surface (e.g. fullscreen or maximized).

        If triggered, the client will receive configure events with the
        "resize" state enum value and the expected sizes. See the "resize" enum
        value for more details about what is required. The client must also
        acknowledge configure events using "ack_configure". After the resize is
        completed, the client will receive another "configure" event without
        the resize state.

        If triggered, the surface also will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the resize.
        It is up to the compositor to visually indicate that the resize is
        taking place, such as updating a pointer cursor, during the resize.
        There is no guarantee that the device focus will return when the resize
        is completed.

        The edges parameter specifies how the surface should be resized, and is
        one of the values of the resize_edge enum. The compositor may use this
        information to update the surface position for example when dragging
        the top left corner. The compositor may also use this information to
        adapt its behavior, e.g. choose an appropriate cursor image.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param edges:
            which edge or corner is being dragged
        :type edges:
            `ArgumentType.Uint`
        """
        self._marshal(6, seat, serial, edges)

    @XdgToplevel.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_max_size(self, width, height):
        """Set the maximum size

        Set a maximum size for the window.

        The client can specify a maximum size so that the compositor does not
        try to configure the window beyond this size.

        The width and height arguments are in window geometry coordinates. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        Values set in this way are double-buffered. They will get applied on
        the next commit.

        The compositor can use this information to allow or disallow different
        states like maximize or fullscreen and draw accurate animations.

        Similarly, a tiling window manager may use this information to place
        and resize client windows in a more effective way.

        The client should not rely on the compositor to obey the maximum size.
        The compositor may decide to ignore the values set by the client and
        request a larger size.

        If never set, or a value of zero in the request, means that the client
        has no expected maximum size in the given dimension. As a result, a
        client wishing to reset the maximum size to an unspecified state can
        use zero for width and height in the request.

        Requesting a maximum size to be smaller than the minimum size of a
        surface is illegal and will result in a protocol error.

        The width and height must be greater than or equal to zero. Using
        strictly negative values for width and height will result in a protocol
        error.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(7, width, height)

    @XdgToplevel.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_min_size(self, width, height):
        """Set the minimum size

        Set a minimum size for the window.

        The client can specify a minimum size so that the compositor does not
        try to configure the window below this size.

        The width and height arguments are in window geometry coordinates. See
        :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`.

        Values set in this way are double-buffered. They will get applied on
        the next commit.

        The compositor can use this information to allow or disallow different
        states like maximize or fullscreen and draw accurate animations.

        Similarly, a tiling window manager may use this information to place
        and resize client windows in a more effective way.

        The client should not rely on the compositor to obey the minimum size.
        The compositor may decide to ignore the values set by the client and
        request a smaller size.

        If never set, or a value of zero in the request, means that the client
        has no expected minimum size in the given dimension. As a result, a
        client wishing to reset the minimum size to an unspecified state can
        use zero for width and height in the request.

        Requesting a minimum size to be larger than the maximum size of a
        surface is illegal and will result in a protocol error.

        The width and height must be greater than or equal to zero. Using
        strictly negative values for width and height will result in a protocol
        error.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(8, width, height)

    @XdgToplevel.request()
    def set_maximized(self):
        """Maximize the window

        Maximize the surface.

        After requesting that the surface should be maximized, the compositor
        will respond by emitting a configure event. Whether this configure
        actually sets the window maximized is subject to compositor policies.
        The client must then update its content, drawing in the configured
        state. The client must also acknowledge the configure when committing
        the new content (see ack_configure).

        It is up to the compositor to decide how and where to maximize the
        surface, for example which output and what region of the screen should
        be used.

        If the surface was already maximized, the compositor will still emit a
        configure event with the "maximized" state.

        If the surface is in a fullscreen state, this request has no direct
        effect. It may alter the state the surface is returned to when
        unmaximized unless overridden by the compositor.
        """
        self._marshal(9)

    @XdgToplevel.request()
    def unset_maximized(self):
        """Unmaximize the window

        Unmaximize the surface.

        After requesting that the surface should be unmaximized, the compositor
        will respond by emitting a configure event. Whether this actually un-
        maximizes the window is subject to compositor policies. If available
        and applicable, the compositor will include the window geometry
        dimensions the window had prior to being maximized in the configure
        event. The client must then update its content, drawing it in the
        configured state. The client must also acknowledge the configure when
        committing the new content (see ack_configure).

        It is up to the compositor to position the surface after it was
        unmaximized; usually the position the surface had before maximizing, if
        applicable.

        If the surface was already not maximized, the compositor will still
        emit a configure event without the "maximized" state.

        If the surface is in a fullscreen state, this request has no direct
        effect. It may alter the state the surface is returned to when
        unmaximized unless overridden by the compositor.
        """
        self._marshal(10)

    @XdgToplevel.request(
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def set_fullscreen(self, output):
        """Set the window as fullscreen on an output

        Make the surface fullscreen.

        After requesting that the surface should be fullscreened, the
        compositor will respond by emitting a configure event. Whether the
        client is actually put into a fullscreen state is subject to compositor
        policies. The client must also acknowledge the configure when
        committing the new content (see ack_configure).

        The output passed by the request indicates the client's preference as
        to which display it should be set fullscreen on. If this value is NULL,
        it's up to the compositor to choose which display will be used to map
        this surface.

        If the surface doesn't cover the whole output, the compositor will
        position the surface in the center of the output and compensate with
        with border fill covering the rest of the output. The content of the
        border fill is undefined, but should be assumed to be in some way that
        attempts to blend into the surrounding area (e.g. solid black).

        If the fullscreened surface is not opaque, the compositor must make
        sure that other screen content not part of the same surface tree (made
        up of subsurfaces, popups or similarly coupled surfaces) are not
        visible below the fullscreened surface.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(11, output)

    @XdgToplevel.request()
    def unset_fullscreen(self):
        """Unset the window as fullscreen

        Make the surface no longer fullscreen.

        After requesting that the surface should be unfullscreened, the
        compositor will respond by emitting a configure event. Whether this
        actually removes the fullscreen state of the client is subject to
        compositor policies.

        Making a surface unfullscreen sets states for the surface based on the
        following: * the state(s) it may have had before becoming fullscreen *
        any state(s) decided by the compositor * any state(s) requested by the
        client while the surface was fullscreen

        The compositor may include the previous window geometry dimensions in
        the configure event, if applicable.

        The client must also acknowledge the configure when committing the new
        content (see ack_configure).
        """
        self._marshal(12)

    @XdgToplevel.request()
    def set_minimized(self):
        """Set the window as minimized

        Request that the compositor minimize your surface. There is no way to
        know if the surface is currently minimized, nor is there any way to
        unset minimization on this surface.

        If you are looking to throttle redrawing when minimized, please instead
        use the :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` event for this, as this
        will also work with live previews on windows in Alt-Tab, Expose or
        similar compositor features.
        """
        self._marshal(13)


class XdgToplevelResource(Resource):
    interface = XdgToplevel

    @XdgToplevel.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Array),
    )
    def configure(self, width, height, states):
        """Suggest a surface change

        This configure event asks the client to resize its toplevel surface or
        to change its state. The configured state should not be applied
        immediately. See :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` for details.

        The width and height arguments specify a hint to the window about how
        its surface should be resized in window geometry coordinates. See
        set_window_geometry.

        If the width or height arguments are zero, it means the client should
        decide its own window dimension. This may happen when the compositor
        needs to configure the state of the surface but doesn't have any
        information about any previous or expected dimension.

        The states listed in the event specify how the width/height arguments
        should be interpreted, and possibly how it should be drawn.

        Clients must send an ack_configure in response to this event. See
        :func:`XdgSurface.configure()
        <pywayland.protocol.xdg_shell.XdgSurface.configure>` and
        :func:`XdgSurface.ack_configure()
        <pywayland.protocol.xdg_shell.XdgSurface.ack_configure>` for details.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        :param states:
        :type states:
            `ArgumentType.Array`
        """
        self._post_event(0, width, height, states)

    @XdgToplevel.event()
    def close(self):
        """Surface wants to be closed

        The close event is sent by the compositor when the user wants the
        surface to be closed. This should be equivalent to the user clicking
        the close button in client-side decorations, if your application has
        any.

        This is only a request that the user intends to close the window. The
        client may choose to ignore this request, or show a dialog to ask the
        user to save their data, etc.
        """
        self._post_event(1)


class XdgToplevelGlobal(Global):
    interface = XdgToplevel


XdgToplevel._gen_c()
XdgToplevel.proxy_class = XdgToplevelProxy
XdgToplevel.resource_class = XdgToplevelResource
XdgToplevel.global_class = XdgToplevelGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSurface
from .zwp_tablet_pad_group_v2 import ZwpTabletPadGroupV2
from .zwp_tablet_v2 import ZwpTabletV2


class ZwpTabletPadV2(Interface):
    """A set of buttons, rings and strips

    A pad device is a set of buttons, rings and strips usually physically
    present on the tablet device itself. Some exceptions exist where the pad
    device is physically detached, e.g. the Wacom ExpressKey Remote.

    Pad devices have no axes that control the cursor and are generally
    auxiliary devices to the tool devices used on the tablet surface.

    A pad device has a number of static characteristics, e.g. the number of
    rings. These capabilities are sent in an event sequence after the
    wp_tablet_seat.pad_added event before any actual events from this pad. This
    initial event sequence is terminated by a wp_tablet_pad.done event.

    All pad features (buttons, rings and strips) are logically divided into
    groups and all pads have at least one group. The available groups are
    notified through the wp_tablet_pad.group event; the compositor will emit
    one event per group before emitting wp_tablet_pad.done.

    Groups may have multiple modes. Modes allow clients to map multiple actions
    to a single pad feature. Only one mode can be active per group, although
    different groups may have different active modes.
    """

    name = "zwp_tablet_pad_v2"
    version = 1

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1


class ZwpTabletPadV2Proxy(Proxy):
    interface = ZwpTabletPadV2

    @ZwpTabletPadV2.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, button, description, serial):
        """Set compositor feedback

        Requests the compositor to use the provided feedback string associated
        with this button. This request should be issued immediately after a
        wp_tablet_pad_group.mode_switch event from the corresponding group is
        received, or whenever a button is mapped to a different action. See
        wp_tablet_pad_group.mode_switch for more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with each button, and compositors may use this
        information to offer visual feedback on the button layout (e.g. on-
        screen displays).

        Button indices start at 0. Setting the feedback string on a button that
        is reserved by the compositor (i.e. not belonging to any
        wp_tablet_pad_group) does not generate an error but the compositor is
        free to ignore the request.

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        wp_tablet_pad_group.mode_switch event received for the group of this
        button. Requests providing other serials than the most recent one will
        be ignored.

        :param button:
            button index
        :type button:
            `ArgumentType.Uint`
        :param description:
            button description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, button, description, serial)

    @ZwpTabletPadV2.request()
    def destroy(self):
        """Destroy the pad object

        Destroy the wp_tablet_pad object. Objects created from this object are
        unaffected and should be destroyed separately.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadV2Resource(Resource):
    interface = ZwpTabletPadV2

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadGroupV2),
    )
    def group(self, pad_group):
        """Group announced

        Sent on wp_tablet_pad initialization to announce available groups. One
        event is sent for each pad group available.

        This event is sent in the initial burst of events before the
        wp_tablet_pad.done event. At least one group will be announced.

        :param pad_group:
        :type pad_group:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadGroupV2`
        """
        self._post_event(0, pad_group)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.String),
    )
    def path(self, path):
        """Path to the device

        A system-specific device path that indicates which device is behind
        this wp_tablet_pad. This information may be used to gather additional
        information about the device, e.g. through libwacom.

        The format of the path is unspecified, it may be a device node, a sysfs
        path, or some other identifier. It is up to the client to identify the
        string provided.

        This event is sent in the initial burst of events before the
        wp_tablet_pad.done event.

        :param path:
            path to local device
        :type path:
            `ArgumentType.String`
        """
        self._post_event(1, path)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
    )
    def buttons(self, buttons):
        """Buttons announced

        Sent on wp_tablet_pad initialization to announce the available buttons.

        This event is sent in the initial burst of events before the
        wp_tablet_pad.done event. This event is only sent when at least one
        button is available.

        :param buttons:
            the number of buttons
        :type buttons:
            `ArgumentType.Uint`
        """
        self._post_event(2, buttons)

    @ZwpTabletPadV2.event()
    def done(self):
        """Pad description event sequence complete

        This event signals the end of the initial burst of descriptive events.
        A client may consider the static description of the pad to be complete
        and finalize initialization of the pad.
        """
        self._post_event(3)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, time, button, state):
        """Physical button state

        Sent whenever the physical state of a button changes.

        :param time:
            the time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param button:
            the index of the button that changed state
        :type button:
            `ArgumentType.Uint`
        :param state:
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(4, time, button, state)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=ZwpTabletV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def enter(self, serial, tablet, surface):
        """Enter event

        Notification that this pad is focused on the specified surface.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param tablet:
            the tablet the pad is attached to
        :type tablet:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletV2`
        :param surface:
            surface the pad is focused on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(5, serial, tablet, surface)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def leave(self, serial, surface):
        """Enter event

        Notification that this pad is no longer focused on the specified
        surface.

        :param serial:
            serial number of the leave event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface the pad is no longer focused on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(6, serial, surface)

    @ZwpTabletPadV2.event()
    def removed(self):
        """Pad removed event

        Sent when the pad has been removed from the system. When a tablet is
        removed its pad(s) will be removed too.

        When this event is received, the client must destroy all rings, strips
        and groups that were offered by this pad, and issue
        wp_tablet_pad.destroy the pad itself.
        """
        self._post_event(7)


class ZwpTabletPadV2Global(Global):
    interface = ZwpTabletPadV2


ZwpTabletPadV2._gen_c()
ZwpTabletPadV2.proxy_class = ZwpTabletPadV2Proxy
ZwpTabletPadV2.resource_class = ZwpTabletPadV2Resource
ZwpTabletPadV2.global_class = ZwpTabletPadV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2015, 2016 Red Hat
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource


class ZwpPrimarySelectionOfferV1(Interface):
    """Offer to transfer primary selection contents

    A wp_primary_selection_offer represents an offer to transfer the contents
    of the primary selection clipboard to the client. Similar to
    :class:`~pywayland.protocol.wayland.WlDataOffer`, the offer also describes
    the mime types that the data can be converted to and provides the
    mechanisms for transferring the data directly to the client.
    """

    name = "zwp_primary_selection_offer_v1"
    version = 1


class ZwpPrimarySelectionOfferV1Proxy(Proxy):
    interface = ZwpPrimarySelectionOfferV1

    @ZwpPrimarySelectionOfferV1.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.FileDescriptor),
    )
    def receive(self, mime_type, fd):
        """Request that the data is transferred

        To transfer the contents of the primary selection clipboard, the client
        issues this request and indicates the mime type that it wants to
        receive. The transfer happens through the passed file descriptor
        (typically created with the pipe system call). The source client writes
        the data in the mime type representation requested and then closes the
        file descriptor.

        The receiving client reads from the read end of the pipe until EOF and
        closes its end, at which point the transfer is complete.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        :param fd:
        :type fd:
            `ArgumentType.FileDescriptor`
        """
        self._marshal(0, mime_type, fd)

    @ZwpPrimarySelectionOfferV1.request()
    def destroy(self):
        """Destroy the primary selection offer

        Destroy the primary selection offer.
        """
        self._marshal(1)
        self._destroy()


class ZwpPrimarySelectionOfferV1Resource(Resource):
    interface = ZwpPrimarySelectionOfferV1

    @ZwpPrimarySelectionOfferV1.event(
        Argument(ArgumentType.String),
    )
    def offer(self, mime_type):
        """Advertise offered mime type

        Sent immediately after creating announcing the
        wp_primary_selection_offer through
        wp_primary_selection_device.data_offer. One event is sent per offered
        mime type.

        :param mime_type:
        :type mime_type:
            `ArgumentType.String`
        """
        self._post_event(0, mime_type)


class ZwpPrimarySelectionOfferV1Global(Global):
    interface = ZwpPrimarySelectionOfferV1


ZwpPrimarySelectionOfferV1._gen_c()
ZwpPrimarySelectionOfferV1.proxy_class = ZwpPrimarySelectionOfferV1Proxy
ZwpPrimarySelectionOfferV1.resource_class = ZwpPrimarySelectionOfferV1Resource
ZwpPrimarySelectionOfferV1.global_class = ZwpPrimarySelectionOfferV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2018 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..xdg_shell import XdgToplevel
from .zxdg_toplevel_decoration_v1 import ZxdgToplevelDecorationV1


class ZxdgDecorationManagerV1(Interface):
    """Window decoration manager

    This interface allows a compositor to announce support for server-side
    decorations.

    A window decoration is a set of window controls as deemed appropriate by
    the party managing them, such as user interface components used to move,
    resize and change a window's state.

    A client can use this protocol to request being decorated by a supporting
    compositor.

    If compositor and client do not negotiate the use of a server-side
    decoration using this protocol, clients continue to self-decorate as they
    see fit.

    Warning! The protocol described in this file is experimental and backward
    incompatible changes may be made. Backward compatible changes may be added
    together with the corresponding interface version bump. Backward
    incompatible changes are done by bumping the version number in the protocol
    and interface names and resetting the interface version. Once the protocol
    is to be declared stable, the 'z' prefix and the version number in the
    protocol and interface names are removed and the interface version number
    is reset.
    """

    name = "zxdg_decoration_manager_v1"
    version = 1


class ZxdgDecorationManagerV1Proxy(Proxy):
    interface = ZxdgDecorationManagerV1

    @ZxdgDecorationManagerV1.request()
    def destroy(self):
        """Destroy the decoration manager object

        Destroy the decoration manager. This doesn't destroy objects created
        with the manager.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgDecorationManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgToplevelDecorationV1),
        Argument(ArgumentType.Object, interface=XdgToplevel),
    )
    def get_toplevel_decoration(self, toplevel):
        """Create a new toplevel decoration object

        Create a new decoration object associated with the given toplevel.

        Creating an xdg_toplevel_decoration from an
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel` which has a buffer
        attached or committed is a client error, and any attempts by a client
        to attach or manipulate a buffer prior to the first
        xdg_toplevel_decoration.configure event must also be treated as errors.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :returns:
            :class:`~pywayland.protocol.xdg_decoration_unstable_v1.ZxdgToplevelDecorationV1`
        """
        id = self._marshal_constructor(1, ZxdgToplevelDecorationV1, toplevel)
        return id


class ZxdgDecorationManagerV1Resource(Resource):
    interface = ZxdgDecorationManagerV1


class ZxdgDecorationManagerV1Global(Global):
    interface = ZxdgDecorationManagerV1


ZxdgDecorationManagerV1._gen_c()
ZxdgDecorationManagerV1.proxy_class = ZxdgDecorationManagerV1Proxy
ZxdgDecorationManagerV1.resource_class = ZxdgDecorationManagerV1Resource
ZxdgDecorationManagerV1.global_class = ZxdgDecorationManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlOutput
from ..wayland import WlSeat


class XdgSurface(Interface):
    """A desktop window

    An interface that may be implemented by a
    :class:`~pywayland.protocol.wayland.WlSurface`, for implementations that
    provide a desktop-style user interface.

    It provides requests to treat surfaces like windows, allowing to set
    properties like maximized, fullscreen, minimized, and to move and resize
    them, and associate metadata like title and app id.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the :class:`XdgSurface`
    state to take effect. Prior to committing the new state, it can set up
    initial configuration, such as maximizing or setting a window geometry.

    Even without attaching a buffer the compositor must respond to initial
    committed configuration, for instance sending a configure event with
    expected window geometry if the client maximized its surface during
    initialization.

    For a surface to be mapped by the compositor the client must have committed
    both an :class:`XdgSurface` state and a buffer.
    """

    name = "xdg_surface"
    version = 1

    class resize_edge(enum.IntEnum):
        none = 0
        top = 1
        bottom = 2
        left = 4
        top_left = 5
        bottom_left = 6
        right = 8
        top_right = 9
        bottom_right = 10

    class state(enum.IntEnum):
        maximized = 1
        fullscreen = 2
        resizing = 3
        activated = 4


class XdgSurfaceProxy(Proxy):
    interface = XdgSurface

    @XdgSurface.request()
    def destroy(self):
        """Destroy the :class:`XdgSurface`

        Unmap and destroy the window. The window will be effectively hidden
        from the user's point of view, and all state like maximization,
        fullscreen, and so on, will be lost.
        """
        self._marshal(0)
        self._destroy()

    @XdgSurface.request(
        Argument(ArgumentType.Object, interface=XdgSurface, nullable=True),
    )
    def set_parent(self, parent):
        """Set the parent of this surface

        Set the "parent" of this surface. This window should be stacked above a
        parent. The parent surface must be mapped as long as this surface is
        mapped.

        Parent windows should be set on dialogs, toolboxes, or other
        "auxiliary" surfaces, so that the parent is raised when the dialog is
        raised.

        :param parent:
        :type parent:
            :class:`XdgSurface` or `None`
        """
        self._marshal(1, parent)

    @XdgSurface.request(
        Argument(ArgumentType.String),
    )
    def set_title(self, title):
        """Set surface title

        Set a short title for the surface.

        This string may be used to identify the surface in a task bar, window
        list, or other user interface elements provided by the compositor.

        The string must be encoded in UTF-8.

        :param title:
        :type title:
            `ArgumentType.String`
        """
        self._marshal(2, title)

    @XdgSurface.request(
        Argument(ArgumentType.String),
    )
    def set_app_id(self, app_id):
        """Set application id

        Set an application identifier for the surface.

        The app ID identifies the general class of applications to which the
        surface belongs. The compositor can use this to group multiple surfaces
        together, or to determine how to launch a new application.

        For D-Bus activatable applications, the app ID is used as the D-Bus
        service name.

        The compositor shell will try to group application surfaces together by
        their app ID.  As a best practice, it is suggested to select app ID's
        that match the basename of the application's .desktop file. For
        example, "org.freedesktop.FooViewer" where the .desktop file is
        "org.freedesktop.FooViewer.desktop".

        See the desktop-entry specification [0] for more details on application
        identifiers and how they relate to well-known D-Bus names and .desktop
        files.

        [0] http://standards.freedesktop.org/desktop-entry-spec/

        :param app_id:
        :type app_id:
            `ArgumentType.String`
        """
        self._marshal(3, app_id)

    @XdgSurface.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def show_window_menu(self, seat, serial, x, y):
        """Show the window menu

        Clients implementing client-side decorations might want to show a
        context menu when right-clicking on the decorations, giving the user a
        menu that they can use to maximize or minimize the window.

        This request asks the compositor to pop up such a window menu at the
        given position, relative to the local surface coordinates of the parent
        surface. There are no guarantees as to what menu items the window menu
        contains.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param x:
            the x position to pop up the window menu at
        :type x:
            `ArgumentType.Int`
        :param y:
            the y position to pop up the window menu at
        :type y:
            `ArgumentType.Int`
        """
        self._marshal(4, seat, serial, x, y)

    @XdgSurface.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
    )
    def move(self, seat, serial):
        """Start an interactive move

        Start an interactive, user-driven move of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive move (touch, pointer, etc).

        The server may ignore move requests depending on the state of the
        surface (e.g. fullscreen or maximized), or if the passed serial is no
        longer valid.

        If triggered, the surface will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the move.
        It is up to the compositor to visually indicate that the move is taking
        place, such as updating a pointer cursor, during the move. There is no
        guarantee that the device focus will return when the move is completed.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(5, seat, serial)

    @XdgSurface.request(
        Argument(ArgumentType.Object, interface=WlSeat),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def resize(self, seat, serial, edges):
        """Start an interactive resize

        Start a user-driven, interactive resize of the surface.

        This request must be used in response to some sort of user action like
        a button press, key press, or touch down event. The passed serial is
        used to determine the type of interactive resize (touch, pointer, etc).

        The server may ignore resize requests depending on the state of the
        surface (e.g. fullscreen or maximized).

        If triggered, the client will receive configure events with the
        "resize" state enum value and the expected sizes. See the "resize" enum
        value for more details about what is required. The client must also
        acknowledge configure events using "ack_configure". After the resize is
        completed, the client will receive another "configure" event without
        the resize state.

        If triggered, the surface also will lose the focus of the device
        (:class:`~pywayland.protocol.wayland.WlPointer`,
        :class:`~pywayland.protocol.wayland.WlTouch`, etc) used for the resize.
        It is up to the compositor to visually indicate that the resize is
        taking place, such as updating a pointer cursor, during the resize.
        There is no guarantee that the device focus will return when the resize
        is completed.

        The edges parameter specifies how the surface should be resized, and is
        one of the values of the resize_edge enum. The compositor may use this
        information to update the surface position for example when dragging
        the top left corner. The compositor may also use this information to
        adapt its behavior, e.g. choose an appropriate cursor image.

        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the user event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :param serial:
            the serial of the user event
        :type serial:
            `ArgumentType.Uint`
        :param edges:
            which edge or corner is being dragged
        :type edges:
            `ArgumentType.Uint`
        """
        self._marshal(6, seat, serial, edges)

    @XdgSurface.request(
        Argument(ArgumentType.Uint),
    )
    def ack_configure(self, serial):
        """Ack a configure event

        When a configure event is received, if a client commits the surface in
        response to the configure event, then the client must make an
        ack_configure request sometime before the commit request, passing along
        the serial of the configure event.

        For instance, the compositor might use this information to move a
        surface to the top left only when the client has drawn itself for the
        maximized or fullscreen state.

        If the client receives multiple configure events before it can respond
        to one, it only has to ack the last configure event.

        A client is not required to commit immediately after sending an
        ack_configure request - it may even ack_configure several times before
        its next surface commit.

        The compositor expects that the most recently received ack_configure
        request at the time of a commit indicates which configure event the
        client is responding to.

        :param serial:
            the serial from the configure event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(7, serial)

    @XdgSurface.request(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_window_geometry(self, x, y, width, height):
        """Set the new window geometry

        The window geometry of a window is its "visible bounds" from the user's
        perspective. Client-side decorations often have invisible portions like
        drop-shadows which should be ignored for the purposes of aligning,
        placing and constraining windows.

        The window geometry is double buffered, and will be applied at the time
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` of the corresponding
        :class:`~pywayland.protocol.wayland.WlSurface` is called.

        Once the window geometry of the surface is set once, it is not possible
        to unset it, and it will remain the same until set_window_geometry is
        called again, even if a new subsurface or buffer is attached.

        If never set, the value is the full bounds of the surface, including
        any subsurfaces. This updates dynamically on every commit. This unset
        mode is meant for extremely simple clients.

        If responding to a configure event, the window geometry in here must
        respect the sizing negotiations specified by the states in the
        configure event.

        The arguments are given in the surface local coordinate space of the
        :class:`~pywayland.protocol.wayland.WlSurface` associated with this
        :class:`XdgSurface`.

        The width and height must be greater than zero.

        :param x:
        :type x:
            `ArgumentType.Int`
        :param y:
        :type y:
            `ArgumentType.Int`
        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        """
        self._marshal(8, x, y, width, height)

    @XdgSurface.request()
    def set_maximized(self):
        """Maximize the window

        Maximize the surface.

        After requesting that the surface should be maximized, the compositor
        will respond by emitting a configure event with the "maximized" state
        and the required window geometry. The client should then update its
        content, drawing it in a maximized state, i.e. without shadow or other
        decoration outside of the window geometry. The client must also
        acknowledge the configure when committing the new content (see
        ack_configure).

        It is up to the compositor to decide how and where to maximize the
        surface, for example which output and what region of the screen should
        be used.

        If the surface was already maximized, the compositor will still emit a
        configure event with the "maximized" state.
        """
        self._marshal(9)

    @XdgSurface.request()
    def unset_maximized(self):
        """Unmaximize the window

        Unmaximize the surface.

        After requesting that the surface should be unmaximized, the compositor
        will respond by emitting a configure event without the "maximized"
        state. If available, the compositor will include the window geometry
        dimensions the window had prior to being maximized in the configure
        request. The client must then update its content, drawing it in a
        regular state, i.e. potentially with shadow, etc. The client must also
        acknowledge the configure when committing the new content (see
        ack_configure).

        It is up to the compositor to position the surface after it was
        unmaximized; usually the position the surface had before maximizing, if
        applicable.

        If the surface was already not maximized, the compositor will still
        emit a configure event without the "maximized" state.
        """
        self._marshal(10)

    @XdgSurface.request(
        Argument(ArgumentType.Object, interface=WlOutput, nullable=True),
    )
    def set_fullscreen(self, output):
        """Set the window as fullscreen on a monitor

        Make the surface fullscreen.

        You can specify an output that you would prefer to be fullscreen. If
        this value is NULL, it's up to the compositor to choose which display
        will be used to map this surface.

        If the surface doesn't cover the whole output, the compositor will
        position the surface in the center of the output and compensate with
        black borders filling the rest of the output.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput` or `None`
        """
        self._marshal(11, output)

    @XdgSurface.request()
    def unset_fullscreen(self):
        """unset_fullscreen
        """
        self._marshal(12)

    @XdgSurface.request()
    def set_minimized(self):
        """Set the window as minimized

        Request that the compositor minimize your surface. There is no way to
        know if the surface is currently minimized, nor is there any way to
        unset minimization on this surface.

        If you are looking to throttle redrawing when minimized, please instead
        use the :func:`WlSurface.frame()
        <pywayland.protocol.wayland.WlSurface.frame>` event for this, as this
        will also work with live previews on windows in Alt-Tab, Expose or
        similar compositor features.
        """
        self._marshal(13)


class XdgSurfaceResource(Resource):
    interface = XdgSurface

    @XdgSurface.event(
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Array),
        Argument(ArgumentType.Uint),
    )
    def configure(self, width, height, states, serial):
        """Suggest a surface change

        The configure event asks the client to resize its surface or to change
        its state.

        The width and height arguments specify a hint to the window about how
        its surface should be resized in window geometry coordinates. See
        set_window_geometry.

        If the width or height arguments are zero, it means the client should
        decide its own window dimension. This may happen when the compositor
        need to configure the state of the surface but doesn't have any
        information about any previous or expected dimension.

        The states listed in the event specify how the width/height arguments
        should be interpreted, and possibly how it should be drawn.

        Clients should arrange their surface for the new size and states, and
        then send a ack_configure request with the serial sent in this
        configure event at some point before committing the new surface.

        If the client receives multiple configure events before it can respond
        to one, it is free to discard all but the last event it received.

        :param width:
        :type width:
            `ArgumentType.Int`
        :param height:
        :type height:
            `ArgumentType.Int`
        :param states:
        :type states:
            `ArgumentType.Array`
        :param serial:
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(0, width, height, states, serial)

    @XdgSurface.event()
    def close(self):
        """Surface wants to be closed

        The close event is sent by the compositor when the user wants the
        surface to be closed. This should be equivalent to the user clicking
        the close button in client-side decorations, if your application has
        any...

        This is only a request that the user intends to close your window. The
        client may choose to ignore this request, or show a dialog to ask the
        user to save their data...
        """
        self._post_event(1)


class XdgSurfaceGlobal(Global):
    interface = XdgSurface


XdgSurface._gen_c()
XdgSurface.proxy_class = XdgSurfaceProxy
XdgSurface.resource_class = XdgSurfaceResource
XdgSurface.global_class = XdgSurfaceGlobal



# Copyright 2021 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time
import threading

import pytest

from pywayland.client.display import Display
from pywayland.client.eventqueue import EventQueue
from pywayland.server.display import Display as ServerDisplay


def _run_server():
    s = ServerDisplay()

    e = s.get_event_loop()
    source = e.add_timer(_kill_server, data=s)
    source.timer_update(500)

    s.add_socket()
    s.run()
    s.destroy()


def _kill_server(data):
    data.terminate()
    return 1


def test_event_queue():
    server = threading.Thread(target=_run_server)
    server.start()

    try:
        display = Display()
        start = time.time()
        while time.time() < start + 1:
            try:
                display.connect()
            except Exception:
                time.sleep(0.1)
            else:
                break
        else:
            pytest.fail("Could not connect to server")

        with display:
            event_queue = EventQueue(display)
            display.dispatch(queue=event_queue)
    finally:
        server.join()



from typing import overload, Any, Callable, TypeVar
from .ffi import (
    CData,
    CharCData,
    ClientCData,
    DispatcherFuncT,
    DisplayCData,
    EventLoopCData,
    InterfaceCData,
    ListCData,
    ListenerCData,
    NotifyFuncT,
    QueueCData,
    ResourceCData,
    ResourceDestroyFuncT,
)

WAYLAND_VERSION_MAJOR: int
WAYLAND_VERSION_MINOR: int
WAYLAND_VERSION_MICRO: int

dispatcher_func: DispatcherFuncT
resource_destroy_func: ResourceDestroyFuncT
notify_func: NotifyFuncT

def wl_list_remove(list: ListCData) -> None: ...

# Event loop functionality
def wl_event_loop_create() -> EventLoopCData: ...
def wl_event_loop_destroy(event_loop: EventLoopCData) -> None: ...

# Display functionality
def wl_display_create() -> DisplayCData: ...
def wl_display_destroy(display: DisplayCData) -> None: ...
def wl_display_destroy_clients(display: DisplayCData) -> None: ...
def wl_display_flush_clients(display: DisplayCData) -> None: ...
def wl_display_get_event_loop(display: DisplayCData) -> EventLoopCData: ...
def wl_display_add_socket(display: DisplayCData, name: bytes) -> int: ...
def wl_display_add_socket_auto(display: DisplayCData) -> CharCData: ...
def wl_display_terminate(display: DisplayCData) -> None: ...
def wl_display_run(display: DisplayCData) -> None: ...
def wl_display_get_serial(display: DisplayCData) -> int: ...
def wl_display_next_serial(display: DisplayCData) -> int: ...
def wl_display_init_shm(display: DisplayCData) -> int: ...
def wl_display_add_shm_format(display: DisplayCData, format: int) -> int: ...
def wl_event_queue_destroy(queue: QueueCData) -> None: ...
def wl_display_connect(name: bytes | CData) -> DisplayCData: ...
def wl_display_connect_to_fd(fd: int) -> DisplayCData: ...
def wl_display_disconnect(display: DisplayCData) -> None: ...
def wl_display_get_fd(display: DisplayCData) -> int: ...
def wl_display_dispatch(display: DisplayCData) -> int: ...
def wl_display_dispatch_pending(display: DisplayCData) -> int: ...
def wl_display_dispatch_queue(display: DisplayCData, queue: QueueCData) -> int: ...
def wl_display_dispatch_queue_pending(
    display: DisplayCData, queue: QueueCData
) -> int: ...
def wl_display_roundtrip(display: DisplayCData) -> int: ...
def wl_display_roundtrip_queue(display: DisplayCData, queue: QueueCData) -> int: ...
def wl_display_get_error(display: DisplayCData) -> int: ...
def wl_display_read_events(display: DisplayCData) -> int: ...
def wl_display_prepare_read(display: DisplayCData) -> int: ...
def wl_display_prepare_read_queue(display: DisplayCData, queue: QueueCData) -> int: ...
def wl_display_flush(display: DisplayCData) -> int: ...
def wl_display_create_queue(display: DisplayCData) -> QueueCData: ...

# Client functionality
def wl_client_create(display: DisplayCData, fd: int) -> ClientCData: ...
def wl_client_destroy(client: ClientCData) -> None: ...
def wl_client_flush(client: ClientCData) -> None: ...
def wl_client_add_destroy_listener(
    client: ClientCData, listener: ListenerCData
) -> None: ...
def wl_client_get_object(client: ClientCData, id: int) -> ResourceCData: ...
def wl_client_get_credentials(
    client: ClientCData, pid: CData, uid: CData, gid: CData
) -> None: ...
def os_create_anonymous_file(size: int) -> int: ...

# Resource functionality
def wl_resource_post_event_array(
    resource: ResourceCData, opcode: int, args: CData
) -> None: ...
def wl_resource_post_error(
    resource: ResourceCData, code: int, msg: bytes, *args: str
) -> None: ...
def wl_resource_create(
    client: ClientCData, interface: InterfaceCData, version: int, id: int
) -> ResourceCData: ...
def wl_resource_set_dispatcher(
    resource: ResourceCData,
    dispatcher: DispatcherFuncT,
    implementation: CData,
    data: CData,
    destroy: ResourceDestroyFuncT,
) -> None: ...
def wl_resource_destroy(resource: ResourceCData) -> None: ...
def wl_resource_get_id(resource: ResourceCData) -> int: ...
def wl_resource_get_user_data(resource: ResourceCData) -> CData: ...
def wl_resource_get_version(resource: ResourceCData) -> int: ...
def wl_resource_add_destroy_listener(
    resource: ResourceCData, listener: ListenerCData
) -> None: ...

WL_EVENT_READABLE: int
WL_EVENT_WRITABLE: int
WL_EVENT_HANGUP: int
WL_EVENT_ERROR: int



# This file has been autogenerated by the pywayland scanner

# Copyright © 2017 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlOutput
from .zxdg_output_v1 import ZxdgOutputV1


class ZxdgOutputManagerV1(Interface):
    """Manage xdg_output objects

    A global factory interface for xdg_output objects.
    """

    name = "zxdg_output_manager_v1"
    version = 3


class ZxdgOutputManagerV1Proxy(Proxy):
    interface = ZxdgOutputManagerV1

    @ZxdgOutputManagerV1.request()
    def destroy(self):
        """Destroy the xdg_output_manager object

        Using this request a client can tell the server that it is not going to
        use the xdg_output_manager object anymore.

        Any objects already created through this instance are not affected.
        """
        self._marshal(0)
        self._destroy()

    @ZxdgOutputManagerV1.request(
        Argument(ArgumentType.NewId, interface=ZxdgOutputV1),
        Argument(ArgumentType.Object, interface=WlOutput),
    )
    def get_xdg_output(self, output):
        """Create an xdg output from a :class:`~pywayland.protocol.wayland.WlOutput`

        This creates a new xdg_output object for the given
        :class:`~pywayland.protocol.wayland.WlOutput`.

        :param output:
        :type output:
            :class:`~pywayland.protocol.wayland.WlOutput`
        :returns:
            :class:`~pywayland.protocol.xdg_output_unstable_v1.ZxdgOutputV1`
        """
        id = self._marshal_constructor(1, ZxdgOutputV1, output)
        return id


class ZxdgOutputManagerV1Resource(Resource):
    interface = ZxdgOutputManagerV1


class ZxdgOutputManagerV1Global(Global):
    interface = ZxdgOutputManagerV1


ZxdgOutputManagerV1._gen_c()
ZxdgOutputManagerV1.proxy_class = ZxdgOutputManagerV1Proxy
ZxdgOutputManagerV1.resource_class = ZxdgOutputManagerV1Resource
ZxdgOutputManagerV1.global_class = ZxdgOutputManagerV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import enum
from weakref import WeakSet
from collections import namedtuple
from typing import List, Optional, TYPE_CHECKING

from pywayland import ffi, lib
from pywayland.utils import ensure_valid

if TYPE_CHECKING:
    from pywayland.server import Display  # noqa: F401

CallbackInfo = namedtuple("CallbackInfo", ["callback", "data"])

# TODO: add error handling to all callbacks


# int (*wl_event_loop_fd_func_t)(int fd, uint32_t mask, void *data)
@ffi.def_extern()
def event_loop_fd_func(fd, mask, data_ptr) -> int:
    callback_info = ffi.from_handle(data_ptr)

    ret = callback_info.callback(fd, mask, callback_info.data)
    if isinstance(ret, int):
        return ret

    return 0


# int (*wl_event_loop_signal_func_t)(int signal_number, void *data)
@ffi.def_extern()
def event_loop_signal_func(signal_number, data_ptr) -> int:
    callback_info = ffi.from_handle(data_ptr)

    ret = callback_info.callback(signal_number, callback_info.data)
    if isinstance(ret, int):
        return ret

    return 0


# int (*wl_event_loop_timer_func_t)(void *data)
@ffi.def_extern()
def event_loop_timer_func(data_ptr):
    callback_info = ffi.from_handle(data_ptr)

    ret = callback_info.callback(callback_info.data)
    if isinstance(ret, int):
        return ret

    return 0


# void (void *data)
@ffi.def_extern()
def event_loop_idle_func(data_ptr):
    callback_info = ffi.from_handle(data_ptr)

    callback_info.callback(callback_info.data)


class EventSource:
    """Parameters for the EventLoop callbacks

    :param cdata: The struct corresponding to the EventSource
    :type cdata: `ffi cdata`
    """

    def __init__(self, eventloop, cdata):
        self._eventloop = eventloop
        self._ptr = cdata

    def remove(self):
        """Remove the callback from the event loop"""
        if self._ptr is not None:
            lib.wl_event_source_remove(self._ptr)
        self._ptr = None

    @ensure_valid
    def check(self):
        """Insert the EventSource into the check list"""
        lib.wl_event_source_check(self._ptr)

    @ensure_valid
    def timer_update(self, timeout):
        """Set the timeout of the times callback

        :param timeout: Delay for timeout in ms
        :type timeout: `int`
        """
        lib.wl_event_source_timer_update(self._ptr, timeout)


class EventLoop:
    """An event loop to add events to

    Returns an event loop.  Either returns the event loop of a given display
    (which will trigger when the Display is run), or creates a new event loop
    (which can be triggered by using :meth:`EventLoop.dispatch()`).

    :param display: The display to create the EventLoop on (default to `None`)
    :type display: :class:`~pywayland.server.Display`
    """

    class FdMask(enum.IntFlag):
        WL_EVENT_READABLE = lib.WL_EVENT_READABLE
        WL_EVENT_WRITABLE = lib.WL_EVENT_WRITABLE
        WL_EVENT_HANGUP = lib.WL_EVENT_HANGUP
        WL_EVENT_ERROR = lib.WL_EVENT_ERROR

    def __init__(self, display: Optional["Display"] = None) -> None:
        if display:
            self._ptr: Optional["ffi.EventLoopCData"] = lib.wl_display_get_event_loop(
                display._ptr
            )
        else:
            # if we are creating an eventloop. we need to destroy it later
            ptr = lib.wl_event_loop_create()
            self._ptr = ffi.gc(ptr, lib.wl_event_loop_destroy)

        self.event_sources: WeakSet[EventSource] = WeakSet()
        self._callback_handles: List = []

    def destroy(self):
        """Destroy the event loop"""
        if self._ptr is not None:
            for event_source in self.event_sources:
                event_source.remove()
            # destroy the pointer and remove the destructor
            ffi.release(self._ptr)
            self._ptr = None

    @ensure_valid
    def add_fd(self, fd, callback, mask=FdMask.WL_EVENT_READABLE, data=None):
        """Add file descriptor callback

        Triggers function call when file descriptor state matches the mask.

        The callback should take three arguments:

            * `fd` - file descriptor (int)
            * `mask` - file descriptor mask (uint)
            * `data` - any object

        :param fd: File descriptor
        :type fd: `int`
        :param callback: Callback function
        :type fd: function with callback `int(int fd, uint32_t mask, void
                  *data)`
        :param mask: File descriptor mask
        :type fd: `int`
        :param data: User data to send to callback
        :type data: `object`
        :returns: :class:`EventSource` for specified callback

        .. seealso::

            :meth:`pywayland.server.eventloop.EventSource.check()`
        """
        callback = CallbackInfo(callback=callback, data=data)
        handle = ffi.new_handle(callback)
        self._callback_handles.append(handle)

        event_source_cdata = lib.wl_event_loop_add_fd(
            self._ptr, fd, mask.value, lib.event_loop_fd_func, handle
        )
        event_source = EventSource(self, event_source_cdata)
        self.event_sources.add(event_source)

        return event_source

    @ensure_valid
    def add_signal(self, signal_number, callback, data=None):
        """Add signal callback

        Triggers function call signal is received.

        The callback should take three arguments:

            * `signal_number` - signal (int)
            * `data` - any object

        :param signal_number: Signal number to trigger on
        :type signal_number: `int`
        :param callback: Callback function
        :type fd: function with callback `int(int signal_number, void *data)`
        :param data: User data to send to callback
        :type data: `object`
        :returns: :class:`EventSource` for specified callback
        """
        callback = CallbackInfo(callback=callback, data=data)
        handle = ffi.new_handle(callback)
        self._callback_handles.append(handle)

        event_source_cdata = lib.wl_event_loop_add_signal(
            self._ptr, signal_number, lib.event_loop_signal_func, handle
        )
        event_source = EventSource(self, event_source_cdata)
        self.event_sources.add(event_source)

        return event_source

    @ensure_valid
    def add_timer(self, callback, data=None):
        """Add timer callback

        Triggers function call after a specified time.

        The callback should take one argument:

            * `data` - any object

        :param callback: Callback function
        :type callback: function with callback `int(void *data)`
        :param data: User data to send to callback
        :type data: `object`
        :returns: :class:`EventSource` for specified callback

        .. seealso::

            :meth:`pywayland.server.eventloop.EventSource.timer_update()`
        """
        callback = CallbackInfo(callback=callback, data=data)
        handle = ffi.new_handle(callback)
        self._callback_handles.append(handle)

        event_source_cdata = lib.wl_event_loop_add_timer(
            self._ptr, lib.event_loop_timer_func, handle
        )
        event_source = EventSource(self, event_source_cdata)
        self.event_sources.add(event_source)

        return event_source

    @ensure_valid
    def add_idle(self, callback, data=None):
        """Add idle callback

        :param callback: Callback function
        :type callback: function with callback `void(void *data)`
        :param data: User data to send to callback
        :returns: :class:`EventSource` for specified callback
        """
        callback = CallbackInfo(callback=callback, data=data)
        handle = ffi.new_handle(callback)
        self._callback_handles.append(handle)

        event_source_cdata = lib.wl_event_loop_add_idle(
            self._ptr, lib.event_loop_idle_func, handle
        )
        event_source = EventSource(self, event_source_cdata)
        self.event_sources.add(event_source)

        return event_source

    @ensure_valid
    def add_destroy_listener(self, listener):
        """Add a listener for the destroy signal

        :param listener: The listener object
        :type listener: :class:`~pywayland.server.Listener`
        """
        lib.wl_event_loop_add_destroy_listener(self._ptr, listener._ptr)

    @ensure_valid
    def dispatch(self, timeout):
        """Dispatch callbacks on the event loop"""
        lib.wl_event_loop_dispatch(self._ptr, timeout)

    @ensure_valid
    def dispatch_idle(self):
        """Dispatch idle callback on the event loop"""
        lib.wl_event_loop_dispatch_idle(self._ptr)



# This file has been autogenerated by the pywayland scanner

# Copyright © 2020 Aleix Pol Gonzalez <aleixpol@kde.org>
# Copyright © 2020 Carlos Garnacho <carlosg@gnome.org>
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from ..wayland import WlSeat
from ..wayland import WlSurface


class XdgActivationTokenV1(Interface):
    """An exported activation handle

    An object for setting up a token and receiving a token handle that can be
    passed as an activation token to another client.

    The object is created using the
    :func:`XdgActivationV1.get_activation_token()
    <pywayland.protocol.xdg_activation_v1.XdgActivationV1.get_activation_token>`
    request. This object should then be populated with the app_id, surface and
    serial information and committed. The compositor shall then issue a done
    event with the token. In case the request's parameters are invalid, the
    compositor will provide an invalid token.
    """

    name = "xdg_activation_token_v1"
    version = 1

    class error(enum.IntEnum):
        already_used = 0


class XdgActivationTokenV1Proxy(Proxy):
    interface = XdgActivationTokenV1

    @XdgActivationTokenV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def set_serial(self, serial, seat):
        """Specifies the seat and serial of the activating event

        Provides information about the seat and serial event that requested the
        token.

        The serial can come from an input or focus event. For instance, if a
        click triggers the launch of a third-party client, the launcher client
        should send a set_serial request with the serial and seat from the
        :func:`WlPointer.button()
        <pywayland.protocol.wayland.WlPointer.button>` event.

        Some compositors might refuse to activate toplevels when the token
        doesn't have a valid and recent enough event serial.

        Must be sent before commit. This information is optional.

        :param serial:
            the serial of the event that triggered the activation
        :type serial:
            `ArgumentType.Uint`
        :param seat:
            the :class:`~pywayland.protocol.wayland.WlSeat` of the event
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        """
        self._marshal(0, serial, seat)

    @XdgActivationTokenV1.request(
        Argument(ArgumentType.String),
    )
    def set_app_id(self, app_id):
        """Specifies the application being activated

        The requesting client can specify an app_id to associate the token
        being created with it.

        Must be sent before commit. This information is optional.

        :param app_id:
            the application id of the client being activated.
        :type app_id:
            `ArgumentType.String`
        """
        self._marshal(1, app_id)

    @XdgActivationTokenV1.request(
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def set_surface(self, surface):
        """Specifies the surface requesting activation

        This request sets the surface requesting the activation. Note, this is
        different from the surface that will be activated.

        Some compositors might refuse to activate toplevels when the token
        doesn't have a requesting surface.

        Must be sent before commit. This information is optional.

        :param surface:
            the requesting surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._marshal(2, surface)

    @XdgActivationTokenV1.request()
    def commit(self):
        """Issues the token request

        Requests an activation token based on the different parameters that
        have been offered through set_serial, set_surface and set_app_id.
        """
        self._marshal(3)

    @XdgActivationTokenV1.request()
    def destroy(self):
        """Destroy the :class:`XdgActivationTokenV1` object

        Notify the compositor that the :class:`XdgActivationTokenV1` object
        will no longer be used.
        """
        self._marshal(4)
        self._destroy()


class XdgActivationTokenV1Resource(Resource):
    interface = XdgActivationTokenV1

    @XdgActivationTokenV1.event(
        Argument(ArgumentType.String),
    )
    def done(self, token):
        """The exported activation token

        The 'done' event contains the unique token of this activation request
        and notifies that the provider is done.

        :param token:
            the exported activation token
        :type token:
            `ArgumentType.String`
        """
        self._post_event(0, token)


class XdgActivationTokenV1Global(Global):
    interface = XdgActivationTokenV1


XdgActivationTokenV1._gen_c()
XdgActivationTokenV1.proxy_class = XdgActivationTokenV1Proxy
XdgActivationTokenV1.resource_class = XdgActivationTokenV1Resource
XdgActivationTokenV1.global_class = XdgActivationTokenV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2013 Kristian Høgsberg
# Copyright © 2013      Rafael Antognolli
# Copyright © 2013      Jasper St. Pierre
# Copyright © 2010-2013 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from pywayland.protocol_core import Global, Interface, Proxy, Resource


class XdgPopup(Interface):
    """Short-lived, popup surfaces for menus

    A popup surface is a short-lived, temporary surface that can be used to
    implement menus. It takes an explicit grab on the surface that will be
    dismissed when the user dismisses the popup. This can be done by the user
    clicking outside the surface, using the keyboard, or even locking the
    screen through closing the lid or a timeout.

    When the popup is dismissed, a popup_done event will be sent out, and at
    the same time the surface will be unmapped. The :class:`XdgPopup` object is
    now inert and cannot be reactivated, so clients should destroy it.
    Explicitly destroying the :class:`XdgPopup` object will also dismiss the
    popup and unmap the surface.

    Clients will receive events for all their surfaces during this grab (which
    is an "owner-events" grab in X11 parlance). This is done so that users can
    navigate through submenus and other "nested" popup windows without having
    to dismiss the topmost popup.

    Clients that want to dismiss the popup when another surface of their own is
    clicked should dismiss the popup using the destroy request.

    The parent surface must have either an
    :class:`~pywayland.protocol.xdg_shell.XdgSurface` or :class:`XdgPopup`
    role.

    Specifying an :class:`XdgPopup` for the parent means that the popups are
    nested, with this popup now being the topmost popup. Nested popups must be
    destroyed in the reverse order they were created in, e.g. the only popup
    you are allowed to destroy at all times is the topmost one.

    If there is an existing popup when creating a new popup, the parent must be
    the current topmost popup.

    A parent surface must be mapped before the new popup is mapped.

    When compositors choose to dismiss a popup, they will likely dismiss every
    nested popup as well. When a compositor dismisses popups, it will follow
    the same dismissing order as required from the client.

    The x and y arguments passed when creating the popup object specify where
    the top left of the popup should be placed, relative to the local surface
    coordinates of the parent surface. See :func:`XdgShell.get_xdg_popup()
    <pywayland.protocol.xdg_shell_unstable_v5.XdgShell.get_xdg_popup>`.

    The client must call :func:`WlSurface.commit()
    <pywayland.protocol.wayland.WlSurface.commit>` on the corresponding
    :class:`~pywayland.protocol.wayland.WlSurface` for the :class:`XdgPopup`
    state to take effect.

    For a surface to be mapped by the compositor the client must have committed
    both the :class:`XdgPopup` state and a buffer.
    """

    name = "xdg_popup"
    version = 1


class XdgPopupProxy(Proxy):
    interface = XdgPopup

    @XdgPopup.request()
    def destroy(self):
        """Remove :class:`XdgPopup` interface

        This destroys the popup. Explicitly destroying the :class:`XdgPopup`
        object will also dismiss the popup, and unmap the surface.

        If this :class:`XdgPopup` is not the "topmost" popup, a protocol error
        will be sent.
        """
        self._marshal(0)
        self._destroy()


class XdgPopupResource(Resource):
    interface = XdgPopup

    @XdgPopup.event()
    def popup_done(self):
        """Popup interaction is done

        The popup_done event is sent out when a popup is dismissed by the
        compositor. The client should destroy the :class:`XdgPopup` object at
        this point.
        """
        self._post_event(0)


class XdgPopupGlobal(Global):
    interface = XdgPopup


XdgPopup._gen_c()
XdgPopup.proxy_class = XdgPopupProxy
XdgPopup.resource_class = XdgPopupResource
XdgPopup.global_class = XdgPopupGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_subsurface import WlSubsurface
from .wl_surface import WlSurface


class WlSubcompositor(Interface):
    """Sub-surface compositing

    The global interface exposing sub-surface compositing capabilities. A
    :class:`~pywayland.protocol.wayland.WlSurface`, that has sub-surfaces
    associated, is called the parent surface. Sub-surfaces can be arbitrarily
    nested and create a tree of sub-surfaces.

    The root surface in a tree of sub-surfaces is the main surface. The main
    surface cannot be a sub-surface, because sub-surfaces must always have a
    parent.

    A main surface with its sub-surfaces forms a (compound) window. For window
    management purposes, this set of
    :class:`~pywayland.protocol.wayland.WlSurface` objects is to be considered
    as a single window, and it should also behave as such.

    The aim of sub-surfaces is to offload some of the compositing work within a
    window from clients to the compositor. A prime example is a video player
    with decorations and video in separate
    :class:`~pywayland.protocol.wayland.WlSurface` objects. This should allow
    the compositor to pass YUV video buffer processing to dedicated overlay
    hardware when possible.
    """

    name = "wl_subcompositor"
    version = 1

    class error(enum.IntEnum):
        bad_surface = 0


class WlSubcompositorProxy(Proxy):
    interface = WlSubcompositor

    @WlSubcompositor.request()
    def destroy(self):
        """Unbind from the subcompositor interface

        Informs the server that the client will not be using this protocol
        object anymore. This does not affect any other objects,
        :class:`~pywayland.protocol.wayland.WlSubsurface` objects included.
        """
        self._marshal(0)
        self._destroy()

    @WlSubcompositor.request(
        Argument(ArgumentType.NewId, interface=WlSubsurface),
        Argument(ArgumentType.Object, interface=WlSurface),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_subsurface(self, surface, parent):
        """Give a surface the role sub-surface

        Create a sub-surface interface for the given surface, and associate it
        with the given parent surface. This turns a plain
        :class:`~pywayland.protocol.wayland.WlSurface` into a sub-surface.

        The to-be sub-surface must not already have another role, and it must
        not have an existing :class:`~pywayland.protocol.wayland.WlSubsurface`
        object. Otherwise a protocol error is raised.

        Adding sub-surfaces to a parent is a double-buffered operation on the
        parent (see :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>`). The effect of adding a
        sub-surface becomes visible on the next time the state of the parent
        surface is applied.

        This request modifies the behaviour of :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` request on the sub-
        surface, see the documentation on
        :class:`~pywayland.protocol.wayland.WlSubsurface` interface.

        :param surface:
            the surface to be turned into a sub-surface
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :param parent:
            the parent surface
        :type parent:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.wayland.WlSubsurface` -- the new sub-
            surface object ID
        """
        id = self._marshal_constructor(1, WlSubsurface, surface, parent)
        return id


class WlSubcompositorResource(Resource):
    interface = WlSubcompositor


class WlSubcompositorGlobal(Global):
    interface = WlSubcompositor


WlSubcompositor._gen_c()
WlSubcompositor.proxy_class = WlSubcompositorProxy
WlSubcompositor.resource_class = WlSubcompositorResource
WlSubcompositor.global_class = WlSubcompositorGlobal



# This file has been autogenerated by the pywayland scanner

# Copyright © 2008-2011 Kristian Høgsberg
# Copyright © 2010-2011 Intel Corporation
# Copyright © 2012-2013 Collabora, Ltd.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource
from .wl_keyboard import WlKeyboard
from .wl_pointer import WlPointer
from .wl_touch import WlTouch


class WlSeat(Interface):
    """Group of input devices

    A seat is a group of keyboards, pointer and touch devices. This object is
    published as a global during start up, or when such a device is hot
    plugged.  A seat typically has a pointer and maintains a keyboard focus and
    a pointer focus.
    """

    name = "wl_seat"
    version = 7

    class capability(enum.IntFlag):
        pointer = 1
        keyboard = 2
        touch = 4

    class error(enum.IntEnum):
        missing_capability = 0


class WlSeatProxy(Proxy):
    interface = WlSeat

    @WlSeat.request(
        Argument(ArgumentType.NewId, interface=WlPointer),
    )
    def get_pointer(self):
        """Return pointer object

        The ID provided will be initialized to the
        :class:`~pywayland.protocol.wayland.WlPointer` interface for this seat.

        This request only takes effect if the seat has the pointer capability,
        or has had the pointer capability in the past. It is a protocol
        violation to issue this request on a seat that has never had the
        pointer capability. The missing_capability error will be sent in this
        case.

        :returns:
            :class:`~pywayland.protocol.wayland.WlPointer` -- seat pointer
        """
        id = self._marshal_constructor(0, WlPointer)
        return id

    @WlSeat.request(
        Argument(ArgumentType.NewId, interface=WlKeyboard),
    )
    def get_keyboard(self):
        """Return keyboard object

        The ID provided will be initialized to the
        :class:`~pywayland.protocol.wayland.WlKeyboard` interface for this
        seat.

        This request only takes effect if the seat has the keyboard capability,
        or has had the keyboard capability in the past. It is a protocol
        violation to issue this request on a seat that has never had the
        keyboard capability. The missing_capability error will be sent in this
        case.

        :returns:
            :class:`~pywayland.protocol.wayland.WlKeyboard` -- seat keyboard
        """
        id = self._marshal_constructor(1, WlKeyboard)
        return id

    @WlSeat.request(
        Argument(ArgumentType.NewId, interface=WlTouch),
    )
    def get_touch(self):
        """Return touch object

        The ID provided will be initialized to the
        :class:`~pywayland.protocol.wayland.WlTouch` interface for this seat.

        This request only takes effect if the seat has the touch capability, or
        has had the touch capability in the past. It is a protocol violation to
        issue this request on a seat that has never had the touch capability.
        The missing_capability error will be sent in this case.

        :returns:
            :class:`~pywayland.protocol.wayland.WlTouch` -- seat touch
            interface
        """
        id = self._marshal_constructor(2, WlTouch)
        return id

    @WlSeat.request(version=5)
    def release(self):
        """Release the seat object

        Using this request a client can tell the server that it is not going to
        use the seat object anymore.
        """
        self._marshal(3)
        self._destroy()


class WlSeatResource(Resource):
    interface = WlSeat

    @WlSeat.event(
        Argument(ArgumentType.Uint),
    )
    def capabilities(self, capabilities):
        """Seat capabilities changed

        This is emitted whenever a seat gains or loses the pointer, keyboard or
        touch capabilities.  The argument is a capability enum containing the
        complete set of capabilities this seat has.

        When the pointer capability is added, a client may create a
        :class:`~pywayland.protocol.wayland.WlPointer` object using the
        :func:`WlSeat.get_pointer()` request. This object will receive pointer
        events until the capability is removed in the future.

        When the pointer capability is removed, a client should destroy the
        :class:`~pywayland.protocol.wayland.WlPointer` objects associated with
        the seat where the capability was removed, using the
        :func:`WlPointer.release()
        <pywayland.protocol.wayland.WlPointer.release>` request. No further
        pointer events will be received on these objects.

        In some compositors, if a seat regains the pointer capability and a
        client has a previously obtained
        :class:`~pywayland.protocol.wayland.WlPointer` object of version 4 or
        less, that object may start sending pointer events again. This behavior
        is considered a misinterpretation of the intended behavior and must not
        be relied upon by the client.
        :class:`~pywayland.protocol.wayland.WlPointer` objects of version 5 or
        later must not send events if created before the most recent event
        notifying the client of an added pointer capability.

        The above behavior also applies to
        :class:`~pywayland.protocol.wayland.WlKeyboard` and
        :class:`~pywayland.protocol.wayland.WlTouch` with the keyboard and
        touch capabilities, respectively.

        :param capabilities:
            capabilities of the seat
        :type capabilities:
            `ArgumentType.Uint`
        """
        self._post_event(0, capabilities)

    @WlSeat.event(
        Argument(ArgumentType.String),
        version=2,
    )
    def name(self, name):
        """Unique identifier for this seat

        In a multiseat configuration this can be used by the client to help
        identify which physical devices the seat represents. Based on the seat
        configuration used by the compositor.

        :param name:
            seat identifier
        :type name:
            `ArgumentType.String`
        """
        self._post_event(1, name)


class WlSeatGlobal(Global):
    interface = WlSeat


WlSeat._gen_c()
WlSeat.proxy_class = WlSeatProxy
WlSeat.resource_class = WlSeatResource
WlSeat.global_class = WlSeatGlobal



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
import xml.etree.ElementTree as ET

from .element import Element
from .printer import Printer

copyright_default = """\
# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License."""


@dataclass(frozen=True)
class Copyright(Element):
    text: str

    @classmethod
    def parse(cls, element: ET.Element) -> "Copyright":
        text = cls.parse_pcdata(element)
        assert text is not None

        return cls(text=text)

    def output(self, printer: Printer) -> None:
        for line in self.text.split("\n"):
            if line:
                printer("# " + line.rstrip())
            else:
                printer("#")
